package main

import (
	"fmt"
	"io/ioutil"
	"log"
	"net/http"
	"os"
	"strings"
	"text/template"

	"gopkg.in/yaml.v3"
)

// Config types
// ------------
type ServerConfig struct {
	Name     string
	Homepage string
}
type PanelConfig struct {
	Host    string
	WebRoot string
	Port    int
	LogPath string
}
type AuthConfig struct {
	GiteaURL        string
	ClientID        string
	ClientSecret    string
	AuthorizedUsers []string
}
type Config struct {
	Server ServerConfig
	Panel  PanelConfig
	Auth   AuthConfig
}

var config = new(Config)
var redirecturi string
var requesturl string

func loadConfig() (err error) {
	configfile, err := ioutil.ReadFile("config.yml")
	if err != nil {
		return err
	}

	err = yaml.Unmarshal(configfile, &config)

	// init oauth bits
	redirecturi = fmt.Sprintf("%s/login/endpoint", config.Panel.Host)
	requesturl = fmt.Sprintf("%s/login/oauth/authorize?client_id=%s&redirect_uri=%s&response_type=code&state=STATE",
		strings.Trim(config.Auth.GiteaURL, " "),
		strings.Trim(config.Auth.ClientID, " "),
		strings.Trim(redirecturi, " "))

	return err
}

func openLogFile(logfile string) {
	if logfile != "" {
		lf, err := os.OpenFile(logfile, os.O_WRONLY|os.O_CREATE|os.O_APPEND, 0640)

		if err != nil {
			log.Fatal("OpenLogfile: os.OpenFile:", err)
		}

		log.SetOutput(lf)
	}
}

func logRequest(handler http.Handler) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		log.Printf("%s %s %s\n", r.RemoteAddr, r.Method, r.URL)
		handler.ServeHTTP(w, r)
	})
}

func exists(path string) (bool, error) {
	_, err := os.Stat(path)
	if err == nil {
		return true, nil
	}
	if os.IsNotExist(err) {
		return false, nil
	}
	return false, err
}

func handler(w http.ResponseWriter, r *http.Request) {
	path := r.URL.Path
	loc := fmt.Sprintf("templates%s", path)

	// var re = regexp.MustCompile(`\.(svg|jpg|jpeg|png|webp|ico|css|js)$`)

	// if re.MatchString(path) {
	// 	http.StripPrefix("/resources/",
	// 		http.FileServer(http.Dir("./resources")))
	// 	return
	// }

	exists, err := exists(loc)
	if err != nil {
		log.Fatal(err)
	}

	if !exists {
		http.NotFound(w, r)
		return
	}

	var tmpl string

	switch path {
	case "/":
		tmpl = "templates/home.html"
		t, _ := template.ParseFiles(tmpl)
		t.Execute(w, config)
	default:
		tmpl = fmt.Sprintf("templates%s", path)
		t, _ := template.ParseFiles(tmpl)
		t.Execute(w, config)
	}
}

func loginHandler(w http.ResponseWriter, r *http.Request) {
	// userid, err := r.Cookie("user-id")
	// if err != nil {
	// 	useridbody := uuid.New()
	// 	expiration := time.Now().Add(365 * 24 * time.Hour)
	// 	cookie := http.Cookie{Name: "user-id", Value: useridbody.String(), Expires: expiration}
	// 	http.SetCookie(w, &cookie)
	// }

	switch r.URL.Path {
	case "/login", "/login/":
		http.Redirect(w, r, requesturl, http.StatusFound)
	case "/login/endpoint", "/login/endpoint/":
		loginEndpoint(w, r)
	}
}

func loginEndpoint(w http.ResponseWriter, r *http.Request) {
	token := r.FormValue("code")
	fmt.Print(token)
}

func main() {
	err := loadConfig()

	if err != nil {
		log.Fatalf("couldn't load config: %s", err)
	}

	openLogFile(config.Panel.LogPath)
	log.SetFlags(log.Ldate | log.Ltime | log.Lshortfile)

	http.HandleFunc("/login/", loginHandler)
	http.Handle("/resources/",
		http.StripPrefix("/resources/",
			http.FileServer(http.Dir("./resources"))))
	http.HandleFunc("/", handler)

	fmt.Printf("listening on %v\n", config.Panel.Port)
	fmt.Printf("Logging to %v\n", config.Panel.LogPath)

	err = http.ListenAndServe(fmt.Sprintf(":%d", config.Panel.Port), logRequest(http.DefaultServeMux))
	if err != nil {
		log.Fatal(err)
	}
}
